% function [fitpars, max_lh] = fit_VPA_model(data)
%
% This function fits the VPA model to a given data set
%
% INPUT
%  data.N    : vector with set size for each trial
%  data.error: vector with response error for each trial (in range [-pi, pi])
%
% If the data set contains only 1 set size, then the power parameter is
% ignored and set to NaN.
%
% This code accompanies the paper "Conceptualizing and testing working 
% memory models in a three-dimensional model space" by Van den Berg, Awh,
% and Ma, published in Psychological Review, 2014.
%
% For questions/bug reports/etc, please email nronaldvdberg@gmail.com

function [fitpars, max_lh, AIC, BIC] = fit_VPA_model(data)

%-%-%-%-%
% init  %
%-%-%-%-%

% discretization of the error space
error_range = linspace(0,pi,91); 
gvar.error_range = error_range(1:end-1)+diff(error_range(1:2))/2;

% mapping between J and kappa
gvar.kappa_max      = 10000;
gvar.kappa_map      = [linspace(0,10,250) linspace(10.001,gvar.kappa_max,250)];
gvar.J_map          = gvar.kappa_map.*besseli(1,gvar.kappa_map,1)./besseli(0,gvar.kappa_map,1);

% settings of evolutionary algorithm used for optimization
gvar.popSizeStart   = 200;                 % size of population at start (Paper: 512)
gvar.popSizeMin     = 32;                  % minimum population size (Paper: 32)
gvar.popSizeRate    = .98;                 % size of new population is popSizeRate times size of current population (unless it is already at minimum)  
gvar.nKids          = 1;                   % number of kids per member 
gvar.nGenerations   = 100;                 % number of generations to simulate (Paper: 256)
gvar.dls            = -0.01;               % difference in ls from generation to generation 
gvar.nMCSamples     = 200;                 % number of MC samples to draw when computing model predictions (Paper: 1000)
gvar.n_par          = 4;                   % number of parameters (J1bar, power, tau, kappa_r)
gvar.samplers       = [2 1 2 2];           % 1=normal, 2=log normal
gvar.ls             = zeros(1,gvar.n_par); % initial width of sampling distributions

% get indices of errors
unique_N = unique(data.N);
for ii=1:length(unique_N)
    trial_idx = find(data.N==unique_N(ii));
    data.error_idx{ii} = interp1(gvar.error_range,1:length(gvar.error_range),abs(data.error_vec(trial_idx)),'nearest','extrap');
end

% initialize matrix with parameter samples with random values
gvar.popSize = gvar.popSizeStart;
X_mat = zeros(gvar.popSize,gvar.n_par);
X_mat(:,1) = 40 + rand(gvar.popSize,1)*20; % J1bar
X_mat(:,2) = -2 + rand(gvar.popSize,1);    % power
X_mat(:,3) = 20 + rand(gvar.popSize,1)*10; % tau
X_mat(:,4) = 40 + rand(gvar.popSize,1)*20; % kappa_r

% if we have only 1 set size, then set power to NaN
if numel(unique_N)==1
    X_mat(:,2) = NaN;
end

% compute fitness of each member in initial generation 
LLH = zeros(1,gvar.popSize);
for jj=1:gvar.popSize
    LLH(jj) = compute_LLH(X_mat(jj,:), data, gvar);
end

% Run genetic algorithm
for ii=1:gvar.nGenerations

    % reproduce!
    X_mat_new = zeros(gvar.popSize*gvar.nKids,gvar.n_par);
    start_idx = 1;
    for jj=1:gvar.popSize
        
        for kk=1:gvar.n_par
            if numel(unique_N)==1 && kk==2
                X_mat_new(start_idx:start_idx+gvar.nKids-1,kk) = NaN; % set power samples to NaN if we have only 1 set size
            else
                X_mat_new(start_idx:start_idx+gvar.nKids-1,kk) = reproduce(X_mat(jj,kk),gvar.ls(kk),gvar.samplers(kk),gvar.nKids);
            end
        end
        start_idx = start_idx + gvar.nKids;
    end
    
    % compute fitness of each member in new generation
    LLH_new = zeros(1,gvar.popSize*gvar.nKids);
    for jj=1:gvar.popSize*gvar.nKids
        LLH_new(jj) = compute_LLH(X_mat_new(jj,:), data, gvar);
    end
    
    % merge old and new 
    X_mat_all = [X_mat; X_mat_new];
    LLH_all = [LLH LLH_new];

    % update population size
    gvar.popSize = max(gvar.popSizeMin,round(gvar.popSizeRate*gvar.popSize));

    % select fittest members (keep population size constant)
    [LLH_all, I] = sort(LLH_all,'descend');    
    LLH = LLH_all(1:gvar.popSize);
    X_mat = X_mat_all(I(1:gvar.popSize),:);
    
    % print some progress information
    if mod(ii,round(gvar.nGenerations/50))==0
        fprintf('.');
    end
    
    gvar.ls = gvar.ls + gvar.dls;
end

% find ML parameter estimates
max_lh = max(LLH);
fitpars = X_mat(LLH==max_lh,:);

% compute AIC and BIC
n_free_pars = 4 - (numel(unique_N)==1);   % Jbar, tau, power, kappa_r  (3 if we have only 1 set size)
BIC = -2*max_lh + n_free_pars*log(numel(data.N));
AIC = -2*max_lh + 2*n_free_pars;

%-%-%-%-%-%-%-%-%-%-%-%
% Likelihood function %
%-%-%-%-%-%-%-%-%-%-%-%
function LLH = compute_LLH(pars, data, gvar)
J1bar = pars(1);
power = pars(2);
tau = pars(3);
kappa_r = pars(4);

N_vec = unique(data.N);  
if numel(N_vec)==1
    power = 0;   % such that Jbar = J1bar
end

% compute model predictions and log likelihood
LLH = 0;
for ii=1:length(N_vec)
    % draw values for J and compute corresponding kappas
    Jbar = ones(1,gvar.nMCSamples)*J1bar*N_vec(ii)^power;
    J = gamrnd(Jbar/tau,tau);
    J = min(J,max(gvar.J_map));
    kappa = interp1(gvar.J_map,gvar.kappa_map,J);
    
    % compute response distribution (add motor noise, marginalize over J)
    kappa_sq = kappa.^2;
    k_c = zeros(length(gvar.error_range),numel(J));
    for jj=1:length(gvar.error_range)
        k_c(jj,:) = sqrt(kappa_r^2 + kappa_sq + 2*kappa_r*kappa*cos(gvar.error_range(jj)));
    end
    p_error = mean(bsxfun(@rdivide,besseli0_fast(k_c,1),2*pi*besseli0_fast(kappa,1)*besseli0_fast(kappa_r,1)).*exp(bsxfun(@minus,k_c,kappa+kappa_r)),2);
       
    % make sure p_error integrates to 0.5 (we're considering only the positive half of the pdf)
    p_error = p_error/sum(p_error) * 1/diff(gvar.error_range(1:2))/2;

    % compute probabilities of reponses, take log, and sum
    p_resp = p_error(data.error_idx{ii});        
    LLH = LLH + sum(log(p_resp));       
end
% this should never happen
if isnan(LLH)
    LLH = -Inf;
end

% Function to sample parameter values
function X_new = reproduce(X_old,ls,sampler_id,n_samples)
if sampler_id==1
    % normal distribution
    X_new = normrnd(X_old,exp(ls),1,n_samples);
elseif sampler_id==2
    % log normal distribution
    X_new = exp(normrnd(log(X_old),exp(ls),1,n_samples));
end