% function data  = gen_fake_VPA_data(parameters,n_trials,N_vec)
%
% Generates synthetic data from the VPA model. 
%
% INPUT
%  modelpars: parameter vector [J1bar power tau kappa_r]
%  n_trials : total number of trials to generate
%  n_vec    : set sizes for which to produce data (uniformly sampled)
%
% If only 1 set size is specified, then the power parameter is automatically 
% set to 0, such that Jbar = J1bar.
%
% This code accompanies the paper "Conceptualizing and testing working 
% memory models in a three-dimensional model space" by Van den Berg, Awh,
% and Ma, published in Psychological Review, 2014.
%
% For questions/bug reports/etc, please email nronaldvdberg@gmail.com

function data  = gen_fake_VPA_data(parameters,n_trials,N_vec)

% parse model pars & flags
J1bar    = parameters(1);
power    = parameters(2);
tau      = parameters(3);
kappa_r  = parameters(4);

% if we have a single set size, set power to 0, such that Jbar=J1bar
if numel(N_vec)==1
    power = 0;
end

% precompute mapping between J and kappa
kmap = linspace(0,700,1e5);
Jmap = kmap.*besseli(1,kmap)./besseli(0,kmap);

% set set size on each trial
data.N = repmat(N_vec,1,n_trials);
data.N = data.N(1:n_trials);

% draw encoding precision and compute corresponding kappa for each trial
Jbar = J1bar*data.N.^power;
J = gamrnd(Jbar/tau,tau);
J = min(J,max(Jmap));
kappa = interp1(Jmap,kmap,J,'linear','extrap');

% draw errors and add motor noise
data.error_vec = circ_vmrnd(0,kappa);   
data.error_vec = circ_vmrnd(data.error_vec,kappa_r);